#include "mex.h" 
#include "math.h"


double DistPow2_Euclidean(double* x1, double* x2,int nx)
{
	double tmp;
	double dist = 0;
	for (int i=0; i<nx; i++)
	{
		tmp = x1[i] - x2[i];
		dist += tmp * tmp;
	}
	return dist;
}

void Encoding(double* x, double* x_Encoded,  double* invsqrtC, double* xmean, int npoints, int nx)	//x'(i) = C^(-0.5) * ( x(i) - xmean(i) )
{
	double *xcur, *xnew;
	double sum;
	int jrow;
	double* dx = (double*)mxCalloc(nx,sizeof(double));
	for (int i=0; i<npoints; i++)
	{
		xcur = &x[i*nx];
		xnew = &x_Encoded[i*nx];
		for (int j=0; j<nx; j++)
			dx[j] = xcur[j] - xmean[j];
		for (int j=0; j<nx; j++)
		{
			sum = 0;
			jrow = j*nx;
			for (int k=0; k<nx; k++)
				sum += invsqrtC[jrow + k] * dx[k];
			xnew[j] = sum;
		}
	}
	mxFree(dx);
}

void EvaluatePoint(double* Fit, double* x_training, double* x_test, int ntest, int nx, int ntrain, double* p_alpha, double TwoSigmaPow2, double* p_Cinv, double* xmean, int doEncoding)
{
	double* x_Encoded = (double*)mxCalloc(nx,sizeof(double));
	double* Kvals = (double*)mxCalloc(ntrain,sizeof(double));

	for(int i=0; i<ntest; i++)
	{
		double* curX = &x_test[i*nx];
		if (doEncoding == 1)
		{	
			Encoding(&x_test[i*nx], x_Encoded, p_Cinv, xmean, 1, nx); 
			curX = x_Encoded;
		}
	

		double distPow2;	
		for (int j=0; j<ntrain; j++)
		{
			distPow2 = DistPow2_Euclidean(curX, &x_training[j*nx], nx);
			Kvals[j] = exp(- distPow2 / TwoSigmaPow2);
		}

		double curFit = 0;
		for (int j=0; j<ntrain-1; j++)
			if (p_alpha[j] != 0)
				curFit += p_alpha[j] *( Kvals[j] - Kvals[j+1] );
		Fit[i] = curFit;
	}	
	mxFree(x_Encoded);
	mxFree(Kvals);
}


//[y_training, y_test] = RankSVMFunc(x_training, x_test, ntest, nx, ntraining, alphas, TwoSigmaPow2, p_Cinv, xmean, doEncoding);
void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
	//input
	double* X_training = mxGetPr(prhs[0]);
	double* X_test = mxGetPr(prhs[1]);
	int ntest = (int)(mxGetScalar(prhs[2]));
	int nx = (int)(mxGetScalar(prhs[3]));				
	int ntraining = (int)(mxGetScalar(prhs[4]));			
	double* p_alpha = mxGetPr(prhs[5]);
	double TwoSigmaPow2 = (double)(mxGetScalar(prhs[6]));	
	double* p_Cinv = mxGetPr(prhs[7]);
	double* xmean = mxGetPr(prhs[8]);
	int doEncoding = (int)(mxGetScalar(prhs[9]));

	int rowLen = mxGetN(prhs[0]);						
	int colLen = mxGetM(prhs[0]);							
	if ((ntraining != rowLen)||(nx != colLen)) 
	{	 // MatLab uses [i*colLen + j] notation, while we use [i*rowLen + j], so .. :) 
		//printf("Error: the matrix 'x_training' should have 'nx' rows and 'ntraining' columns");
		return;
	}
	//output
	plhs[0] = mxCreateDoubleMatrix(1,ntest,  mxREAL);
	double* Fit = mxGetPr(plhs[0]);

	EvaluatePoint(Fit, X_training,X_test, ntest,nx, ntraining, p_alpha, TwoSigmaPow2, p_Cinv, xmean, doEncoding);
}
