#include "mex.h" 
#include "math.h"

enum KernelType
{
	kt_RBF_Euclidean = 0,
	kt_RBF_Mahalanobis = 1
};

void OptimizeL(int ntrain, double* p_Ci, double epsilon, int niter,
						 double* p_Kij,double* p_dKij,double* p_alpha,double* p_sumAlphaDKij,double* p_div_dKij)
{
	int nAlpha = ntrain-1;
	double old_alpha, new_alpha, delta_alpha, sumAlpha, dL;
	int i,i1,j;

	for (i=0; i<nAlpha; i++)
		for (j=0; j<nAlpha; j++)
			p_dKij[i*nAlpha + j] = p_Kij[i*ntrain + j] - p_Kij[i*ntrain + (j+1)] - p_Kij[(i+1)*ntrain + j] + p_Kij[(i+1)*ntrain + (j+1)];
	
	for (i=0; i<nAlpha;i++)
	{
	//	p_alpha[i] = p_Ci[i];// * rand()/(float)RAND_MAX;//p_Ci[i] * (0.95 + 0.05*rand()/(float)RAND_MAX);	// p_Ci[i] * rand()/(float)RAND_MAX;
	//	p_alpha[i] = p_Ci[i] * rand()/(float)RAND_MAX;
		p_alpha[i] = p_Ci[i] * (0.95 + 0.05*rand()/(float)RAND_MAX);
	}

	for (i=0; i<nAlpha; i++)
	{
		sumAlpha = 0;
		for (j=0; j<nAlpha;j++)
			sumAlpha += p_alpha[j] * p_dKij[i*nAlpha + j];
		p_sumAlphaDKij[i] = -(sumAlpha - epsilon) / p_dKij[i*nAlpha + i];
	}

	for (i=0; i<nAlpha; i++)
		for (j=0; j<nAlpha; j++)
			p_div_dKij[i*nAlpha + j] = p_dKij[i*nAlpha + j] / p_dKij[j*nAlpha + j];

	for (i=0; i<niter; i++)
	{	
		i1 = i%nAlpha;	//	int i1 = rand()%nAlpha;
		old_alpha = p_alpha[i1];
		new_alpha = old_alpha + p_sumAlphaDKij[i1];
		if (new_alpha > p_Ci[i1])		new_alpha = p_Ci[i1];
		if (new_alpha < 0)				new_alpha = 0;
		delta_alpha = new_alpha - old_alpha;

		dL = delta_alpha * p_dKij[i1*nAlpha + i1] * ( p_sumAlphaDKij[i1] - 0.5*delta_alpha + epsilon);

		if (dL > 0)
		{
			for (j=0; j<nAlpha; j++)
				p_sumAlphaDKij[j] -= delta_alpha * p_div_dKij[i1*nAlpha + j];

			p_alpha[i1] = new_alpha;
		}
	}
}

double DistPow2_Euclidean(double* x1, double* x2,int nx)
{
	double tmp;
	double dist = 0;
	for (int i=0; i<nx; i++)
	{
		tmp = x1[i] - x2[i];
		dist += tmp * tmp;
	}
	return dist;
}

void Encoding(double* x, double* x_Encoded, double* invsqrtC, double* xmean, int npoints, int nx)	//x'(i) = C^(-0.5) * ( x(i) - xmean(i) )
{
	double *xcur, *xnew;
	double sum;
	int jrow;
	double* dx = (double*)mxCalloc(nx,sizeof(double));
	for (int i=0; i<npoints; i++)
	{
		xcur = &x[i*nx];
		xnew = &x_Encoded[i*nx];
		for (int j=0; j<nx; j++)
			dx[j] = xcur[j] - xmean[j];
		for (int j=0; j<nx; j++)
		{
			sum = 0;
			jrow = j*nx;
			for (int k=0; k<nx; k++)
				sum += invsqrtC[jrow + k] * dx[k];
			xnew[j] = sum;
		}
	}
	mxFree(dx);
}

void Learning(double* x_training, double* x_trainingEncoded,   
		int nx, int ntrain, int niter, KernelType kernel, double epsilon, double* p_Ci, double* p_Cinv,
		double sigma_A, double sigma_Pow, double* xmean, int doEncoding, double* optAlphas, double* pTwoSigmaPow2)

{
	//0. Init Temp Data
	int nAlpha = ntrain-1;	
	double* p_Kij = (double*)mxCalloc(ntrain*ntrain,sizeof(double));	double* p_dKij = (double*)mxCalloc(nAlpha*nAlpha,sizeof(double));
	double* p_alpha = (double*)mxCalloc(nAlpha,sizeof(double));		double* p_sumAlphaDKij = (double*)mxCalloc(nAlpha,sizeof(double));
	double* p_div_dKij = (double*)mxCalloc(nAlpha*nAlpha,sizeof(double));	double* p_dx = (double*)mxCalloc(nx,sizeof(double));
	double* Kvals = (double*)mxCalloc(ntrain,sizeof(double));

	double ttotal = 0;
	//1. Transform training points to the new coordinate system and 
	//then compute Euclidean distance instead of 'EXPENSIVE' Mahalanobis distance
	if (doEncoding == 1)
	{	//p_Cinv is the C^-0.5 ;)
		Encoding(x_training, x_trainingEncoded, p_Cinv, xmean, ntrain, nx); 
	}
	
	//2. Calculate the distance between points, then calculate sigma(gamma) and Kernel Matrix
	double distPow2;
	double avrdist = 0;
	for (int i=0; i<ntrain; i++)
	 for (int j=i; j<ntrain; j++)
		if (i == j)		p_Kij[i*ntrain + j] = 0;
		else
		{
			distPow2 = DistPow2_Euclidean(&x_trainingEncoded[i*nx], &x_trainingEncoded[j*nx], nx);
			avrdist += sqrt(distPow2);
			p_Kij[i*ntrain + j] = p_Kij[j*ntrain + i] = distPow2;//dist;
		}
	avrdist = avrdist / ((ntrain-1)*ntrain/2);		//average distance
	double sigma = sigma_A * pow(avrdist , sigma_Pow);
	double TwoSigmaPow2 = 2.0*sigma*sigma;
	for (int i=0; i<ntrain; i++)
		for (int j=i; j<ntrain; j++)
		{
			if (i == j)		p_Kij[i*ntrain + j] = 1.0;
			else			p_Kij[i*ntrain + j] = p_Kij[j*ntrain + i] = exp(- p_Kij[i*ntrain + j] / TwoSigmaPow2);
		}


	//3. Optimize alpha parameters
	OptimizeL(ntrain, p_Ci, epsilon, niter, p_Kij, p_dKij, p_alpha, p_sumAlphaDKij, p_div_dKij);

	
	*pTwoSigmaPow2 = TwoSigmaPow2;
	for (int i=0; i<nAlpha; i++)
	{
		optAlphas[i] = p_alpha[i];
	}

	mxFree(p_sumAlphaDKij);		mxFree(p_div_dKij);		mxFree(p_dKij);		mxFree(p_Kij);
	mxFree(p_dx);	mxFree(p_alpha);	mxFree(Kvals);
}



//[alphas, y_test] = RankSVM(x_training', nx, ntraining,  niter, epsilon, Ci, kernel, Cinv, sigmaA, sigmaPow, xmean, doEncoding, verbose);
void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
	//input
	int verbose = (int)(mxGetScalar(prhs[12]));

	double* X_training = mxGetPr(prhs[0]);
	int nx = (int)(mxGetScalar(prhs[1]));				//if (verbose == 1) printf("nx = %d\n",nx);
	int ntraining = (int)(mxGetScalar(prhs[2]));			//if (verbose == 1) printf("ntraining = %d\n",ntraining);
	int niter = (int)(mxGetScalar(prhs[3]));			//if (verbose == 1) printf("niter = %d\n",niter);
	double epsilon = (double)(mxGetScalar(prhs[4]));		//if (verbose == 1) printf("epsilon = %f\n",epsilon);
	double* p_Ci = mxGetPr(prhs[5]);
	int kernel = (int)(mxGetScalar(prhs[6]));			//if (verbose == 1) printf("kernel = %d\n",kernel);
	double* p_Cinv = mxGetPr(prhs[7]);
	double sigma_A = (double)(mxGetScalar(prhs[8]));		//if (verbose == 1) printf("sigma_A = %f\n",sigma_A);
	double sigma_Pow = (double)(mxGetScalar(prhs[9]));		//if (verbose == 1) printf("sigma_Pow = %f\n",sigma_Pow);
	double* xmean = mxGetPr(prhs[10]);
	int doEncoding = (int)(mxGetScalar(prhs[11]));			//if (verbose == 1) printf("doEncoding = %d\n",doEncoding);
	//
	int rowLen = mxGetN(prhs[0]);					//if (verbose == 1) printf("rowLen=%d\n",rowLen);
	int colLen = mxGetM(prhs[0]);					//if (verbose == 1) printf("colLen=%d\n",colLen);
	if ((ntraining != rowLen)||(nx != colLen)) 
	{	 // MatLab uses [i*colLen + j] notation, while we use [i*rowLen + j], so .. :) 
		//printf("Error: the matrix 'x_training' should have 'nx' rows and 'ntraining' columns");
		return;
	}

	//output
	plhs[0] = mxCreateDoubleMatrix(nx, ntraining, mxREAL);
	double* X_trainingEncoded = mxGetPr(plhs[0]);

	plhs[1] = mxCreateDoubleMatrix(ntraining-1, 1, mxREAL);
	double* optAlphas = mxGetPr(plhs[1]);

	plhs[2] = mxCreateDoubleMatrix(1, 1, mxREAL);
	double* pTwoSigmaPow2 = mxGetPr(plhs[2]);

	Learning(X_training, X_trainingEncoded, nx, ntraining, niter, (KernelType)kernel, epsilon, p_Ci, p_Cinv, sigma_A, sigma_Pow, xmean, doEncoding, optAlphas, pTwoSigmaPow2);
}
